{% if record.is_check %}
    {% if record.type == 1 or record.type == 2 %}
        <a href="{% url 'bee_django_user:leave_cancel' record.id %}" class="btn btn-default">销假</a>
    {% endif %}
{% endif %}